import tirmite as tsplit
import argparse
import shutil

def mainArgs():
	parser = argparse.ArgumentParser(
							description	=	'Extract terminal repeats from retrotransposons (LTRs) or DNA transposons (TIRs). \
											Optionally, compose synthetic MITES from complete DNA transposons.',
							prog		=	'exterminate'
							)
	parser.add_argument('-i', '--infile',
							type		=	str,
							required	=	True,
							default		=	None,
							help		=	'Multifasta containing complete elements.'
							)
	parser.add_argument('-p', '--prefix',
							type	=	str,
							default	=	None,
							help	=	'All output files begin with this string. \
										(Default: [infile name])'
							)
	parser.add_argument('-d', '--outdir',
							type	=	str,
							default	=	None,
							help	=	'Write output files to this directory. \
										(Default: cwd)'
							)
	parser.add_argument('--splitmode',
							default	=	'split',
							choices	=	['all','split','internal','external',None],
							help	=	'all= Report input sequence as well as internal and external segments. \
										split= Report internal and external segments after splitting. \
										internal = Report only internal segments \
										external = Report only terminal repeat segments. \
										If set to "None" then only synthetic MITES will be reported if --makemites is also set. \
										(Default: split)'
							)
	parser.add_argument('--keeptemp',
							action	=	'store_true',
							default	=	False,
							help	=	'If set do not remove temp directory on completion.'
							)
	parser.add_argument('-v','--verbose',
							action	=	'store_true',
							default	=	False,
							help	=	'If set, report progress.'
							)
	parser.add_argument('-m', '--maxdist',
							type	=	int,
							default	=	10,
							help	=	'Terminal repeat candidates must be no more than this many bases from end of input element. (Default: 10)\
										Note: Increase this value if you suspect that your element is nested within some flanking sequence.'
							)
	parser.add_argument('--minid',
							type	=	float,
							default	=	80.0,
							help	=	'Minimum identity between terminal repeat pairs. As float. \
										(Default: 80.0)'
							)
	parser.add_argument('--minterm',
							type	=	int,
							default	=	10,
							help	=	'Minimum length for a terminal repeat to be considered. \
										Equivalent to nucmer "--mincluster" \
										(Default: 10)'
							)
	parser.add_argument('--minseed',
							type	=	int,
							default	=	5,
							help	=	'Minimum length of a maximal exact match to be included in final match cluster. \
										Equivalent to nucmer "--minmatch". \
										(Default: 5)'
							)
	parser.add_argument('--diagfactor',
							type	=	float,
							default	=	0.2,
							help	=	'Maximum diagonal difference factor for clustering of matches within nucmer, \
										i.e. diagonal difference / match separation (default 0.20) \
										Note: Increase value for greater tolerance of indels between terminal repeats.'
							)
	parser.add_argument('--method',
							default	=	'nucmer',
							choices	=	['blastn','nucmer'],
							help	=	'Select alignment method: "blastn" or "nucmer".(Default: nucmer)'
							)
	args = parser.parse_args()
	return args

def missing_tool(tool_name):
    path = shutil.which(tool_name)
    if path is None:
        return [tool_name]
    else:
        return []

def main():
	'''Do the work.'''
	# Get cmd line args
	args = mainArgs()

	# Check for required programs.
	tools = ['delta-filter','nucmer','show-coords']
	if args.method == 'blastn':
		tools.append('blastn')
	missing_tools = []
	for tool in tools:
	    missing_tools += missing_tool(tool)
	if missing_tools:
	    print('WARNING: Some tools required by Exterminate could not be found: ' +
	          ', '.join(missing_tools))
	    print('You may need to install them to use all features.')

	# Create output paths as required
	outpath,tempdir = tsplit.tSplitchecks(args)
	
	# Load elements to be screened
	elements = tsplit.importFasta2List(args.infile)
	
	# If LTR mode, search for terminal repeats on same strand
	segments = tsplit.getLTRs( elements	=	elements,
						flankdist	=	args.maxdist,
						minterm		=	args.minterm,
						minseed		=	args.minseed,
						minid		=	args.minid,
						diagfactor	=	args.diagfactor,
						report		=	args.splitmode,
						temp		=	tempdir,
						alignTool	=	args.method,
						keeptemp	=	args.keeptemp,
						verbose		=	args.verbose
						)
	tsplit.segWrite(outpath, segs=segments)

	# Remove temp directory
	if not args.keeptemp:
		shutil.rmtree(tempdir)



import os
import sys
import shutil
import tempfile
import subprocess
from shlex import quote

class Error (Exception): pass

def decode(x):
	try:
		s = x.decode()
	except:
		return x
	return s

def _write_script(cmds,script):
	'''Write commands into a bash script'''
	f = open(script, 'w+')
	for cmd in cmds:
		print(cmd, file=f)
	f.close()

def syscall(cmd, verbose=False):
	'''Manage error handling when making syscalls'''
	if verbose:
		print('Running command:', cmd, flush=True)
	try:
		output = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
	except subprocess.CalledProcessError as error:
		print('The following command failed with exit code', error.returncode, file=sys.stderr)
		print(cmd, file=sys.stderr)
		print('\nThe output was:\n', file=sys.stderr)
		print(error.output.decode(), file=sys.stderr)
		raise Error('Error running command:', cmd)
	if verbose:
		print(decode(output))

def makeBlast(seq=None, outfile=None, pid=60):
	cmd = 'blastn -word_size 4 -outfmt "6 qstart qend sstart send length positive pident qlen slen qframe sframe qseqid sseqid" -query ' + quote(str(seq)) + ' -subject ' + quote(str(seq)) + ' -out ' + quote(str(outfile)) + ' -perc_identity ' + str(pid)
	return [cmd]

def run_blast(cmds,verbose=False):
	'''Write and excute HMMER script'''
	tmpdir = tempfile.mkdtemp(prefix='tmp.', dir=os.getcwd())
	original_dir = os.getcwd()
	os.chdir(tmpdir)
	script = 'run_jobs.sh'
	_write_script(cmds,script)
	syscall('bash ' + script, verbose=verbose)
	os.chdir(original_dir)
	shutil.rmtree(tmpdir)


'''
Recreate pymummer-like coords file output from blast.

#Blast field, coords field, Description

qstart			[S1] 	Start of the alignment region in the reference sequence 
qend			[E1] 	End of the alignment region in the reference sequence 
sstart			[S2] 	Start of the alignment region in the query sequence 
send 			[E2] 	End of the alignment region in the query sequence 
length			[LEN 1] Length of the alignment region in the reference sequence
positive		[LEN 2] Length of the alignment region in the query sequence (#"positive" is just a filler as blast won't all repeated fields)
pident			[% IDY] Percent identity of the alignment 
qlen			[LEN R] Length of the reference sequence 
slen			[LEN Q] Length of the query sequence 
qframe sframe	[FRM] 	Reading frame for the reference AND query sequence alignments respectively 
qseqid sseqid	[TAGS] 	The reference AND query FastA IDs respectively. All output coordinates and lengths are relative to the forward strand of the reference DNA sequence.
'''